import 'package:flutter/material.dart';

enum BaseTextType {
  h1,
  h2,
  h3,
  h4,
  h5,
  h6,
  subtitle1,
  subtitle2,
  body1,
  body2,
  caption,
  overline
}

class Label extends StatelessWidget {
  final String text;
  final BaseTextType type;
  final Color color;
  final TextAlign textAlign;
  final int maxLines;
  final bool softWrap;
  final int fontWeightDelta;
  final TextOverflow overflow;
  final FontStyle fontStyle;

  Label(this.text,
      {this.type,
      this.color,
      this.textAlign = TextAlign.left,
      this.maxLines,
      this.softWrap = true,
      this.overflow,
      this.fontWeightDelta = 1,
      this.fontStyle = FontStyle.normal});

  @override
  Widget build(BuildContext context) {
    var textTheme = Theme.of(context).textTheme;
    var textStyle;
    switch (type) {
      case BaseTextType.h1:
        textStyle = textTheme.headline1
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.h2:
        textStyle = textTheme.headline2
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.h3:
        textStyle = textTheme.headline3
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.h4:
        textStyle = textTheme.headline4
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.h5:
        textStyle = textTheme.headline5
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.h6:
        textStyle = textTheme.headline6
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.subtitle1:
        textStyle = textTheme.subtitle1
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.subtitle2:
        textStyle = textTheme.subtitle2
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.body1:
        textStyle = textTheme.bodyText2
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.body2:
        textStyle = textTheme.bodyText1
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.caption:
        textStyle = textTheme.caption
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      case BaseTextType.overline:
        textStyle = textTheme.overline
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
      default:
        textStyle = DefaultTextStyle.of(context)
            .style
            .apply(fontWeightDelta: fontWeightDelta, fontStyle: fontStyle);
        break;
    }
    if (color != null) {
      textStyle = textStyle.copyWith(color: color);
    }
    return Text(
      text,
      style: textStyle,
      textAlign: textAlign,
      maxLines: maxLines,
      overflow: TextOverflow.ellipsis,
      softWrap: softWrap,
    );
  }
}
