import 'package:flutter/material.dart';
import 'package:multi_image_picker/multi_image_picker.dart';

import '../../mobile_ui.dart';
import '../../mobile_utils.dart';
import 'form_input_base.dart';

class MultiImagePickerFormInput extends FormInputBase<List<Asset>> {
  final int maxImage;
  final int thumbSize;

  MultiImagePickerFormInput(
      {Key key,
      FormFieldSetter<List<Asset>> onSaved,
      ValueValidator<List<Asset>> validator,
      String labelText,
      this.maxImage = 6,
      this.valueController,
      this.thumbSize = 90,
      bool isRequired = false,
      bool readOnly = false,
      bool enabled = true,
      List<DropdownMenuItem<List<Asset>>> items,
      List<Asset> initialValue,
      AutovalidateMode autovalidateMode,
      TextAlign labelAlign = TextAlign.left,
      FieldOrientation orientation = FieldOrientation.horizontal,
      EdgeInsetsGeometry contentPadding = const EdgeInsets.all(8),
      EdgeInsetsGeometry margin = const EdgeInsets.symmetric(vertical: 8),
      ControlPosition controlPosition = ControlPosition.trailing})
      : super(
            initialValue: initialValue,
            isRequired: isRequired,
            onSaved: onSaved,
            validator: validator,
            enabled: enabled,
            autovalidateMode: autovalidateMode,
            orientation: orientation,
            controlPosition: controlPosition,
            labelText: labelText,
            labelAlign: labelAlign,
            valueController: valueController,
            contentPadding: contentPadding,
            builder: (field) {
              MultiImagePickerFormInput widget = field.widget;
              final _MultiImagePickerFormFieldState state = field;
              final theme = Theme.of(field.context);
              return widget.buildField(field, state.buildGridView(theme));
            });

  final ValueController<List<Asset>> valueController;

  @override
  _MultiImagePickerFormFieldState createState() =>
      _MultiImagePickerFormFieldState();
}

class _MultiImagePickerFormFieldState extends FormInputStateBase<List<Asset>> {
  @override
  MultiImagePickerFormInput get widget => super.widget;

  Widget buildGridView(ThemeData theme) {
    var children = buildImages(theme);
    final color = theme.brightness == Brightness.light
        ? Colors.grey.shade700
        : Colors.white70;
    children.add(InkWell(
        onTap: () => loadAssets(theme),
        child: Container(
            alignment: Alignment.center,
            width: widget.thumbSize.toDouble(),
            height: widget.thumbSize.toDouble(),
            decoration: BoxDecoration(
              border: Border.all(color: color),
            ),
            child: Column(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: <Widget>[
                Icon(
                  Icons.camera_enhance,
                  size: 40,
                ),
                Text(
                  "Upload images",
                  style: TextStyle(fontSize: 12),
                )
              ],
            ))));
    return Padding(
        padding: EdgeInsets.only(top: 10, bottom: 20),
        child: Wrap(
          children: children,
          spacing: 5,
          runSpacing: 5,
        ));
  }

  List<Widget> buildImages(ThemeData theme) {
    if (value != null) {
      return List.generate(value.length, (index) {
        var asset = value[index];
        return Stack(alignment: AlignmentDirectional.bottomEnd, children: [
          AssetThumb(
            asset: asset,
            width: widget.thumbSize,
            height: widget.thumbSize,
          ),
          widget.buildIcon(theme, Icons.delete_outline, onTap: () {
            var resultList = value;
            resultList.remove(asset);
            changeValue(resultList);
          }, enabledColor: Colors.red),
        ]);
      });
    }
    return <Widget>[];
  }

  Future<void> loadAssets(ThemeData theme) async {
    List<Asset> resultList;
    var color =
        '#${theme.primaryColor.value.toRadixString(16).padLeft(8, '0').substring(2)}';
    try {
      resultList = await MultiImagePicker.pickImages(
        maxImages: widget.maxImage,
        enableCamera: true,
        cupertinoOptions: CupertinoOptions(takePhotoIcon: "chat"),
        materialOptions: MaterialOptions(
          actionBarColor: color,
          actionBarTitle: "Upload Images",
          allViewTitle: "All Photos",
          startInAllView: true,
          textOnNothingSelected: null,
          useDetailsView: false,
        ),
      );
    } on Exception catch (e) {
      showError(e.toString());
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;
    if (resultList != null) {
      resultList.insertAll(0, value);
      didChange(resultList);
    }
  }
}
