import 'package:flutter/material.dart';

import '../../mobile_ui.dart';

class SwitchFormInput extends FormInputBase<bool> {
  SwitchFormInput(
      {Key key,
      String labelText,
      bool initialValue,
      ValueController<bool> controller,
      FocusNode focusNode,
      bool readOnly = false,
      bool enabled = true,
      bool isRequired = false,
      FormFieldSetter<bool> onSaved,
      ValueChanged<bool> onChanged,
      AutovalidateMode autovalidateMode,
      TextAlign labelAlign = TextAlign.left,
      ValueValidator<bool> validator,
      FieldOrientation orientation = FieldOrientation.horizontal,
      EdgeInsetsGeometry contentPadding = const EdgeInsets.all(0),
      EdgeInsetsGeometry margin = const EdgeInsets.symmetric(vertical: 8),
      ControlPosition controlPosition = ControlPosition.trailing})
      : super(
          key: key,
          initialValue: controller != null ? null : (initialValue ?? false),
          onSaved: onSaved,
          validator: validator,
          enabled: enabled,
          autovalidateMode: autovalidateMode,
          orientation: orientation,
          controlPosition: controlPosition,
          labelText: labelText,
          labelAlign: labelAlign,
          valueController: controller,
          isRequired: isRequired,
          contentPadding: contentPadding,
          // ignore: missing_return
          builder: (field) {
            FormInputBase<bool> widget = field.widget;
            FormInputStateBase<bool> state = field;
            var switchButton = Switch(
                focusNode: focusNode,
                onChanged: enabled == true
                    ? (value) {
                        if (readOnly == true) return;
                        state.changeValue(value);
                        if (onChanged != null) onChanged(value);
                      }
                    : null,
                value: state.effectiveValueController.value,
                materialTapTargetSize: MaterialTapTargetSize.shrinkWrap);
            return widget.buildField(field, switchButton);
          },
        );
}
